/*
 * Copyright (C) 1997-2005 Andre Noll <maan@systemlinux.org>
 *
 *     This program is free software; you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation; either version 2 of the License, or
 *     (at your option) any later version.
 *
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with this program; if not, write to the Free Software
 *     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
 */
#include "para.h"
#include <pwd.h>
/*
 * do the usual stuff to become a daemon
 */
void daemon_init(void)
{
	pid_t pid;

	PARA_INFO_LOG("%s", "daemonizing\n");
	if ((pid = fork()) < 0) {
		PARA_EMERG_LOG("%s", "failed to init daemon\n");
		exit(EXIT_FAILURE);
	} else if (pid)
		exit(EXIT_SUCCESS); /* parent exits */
	/* child */
	setsid(); /* become session leader */
	chdir("/");
	umask(0);
}

/*
 * call exit() on errors this might be called from para_log, so we must not
 * use para_log...
 */
FILE *open_log(char *logfile_name)
{
	FILE *logfile;

	if (!logfile_name)
		return NULL;
	if (!(logfile = fopen(logfile_name, "a")))
		exit(EXIT_FAILURE);
	setlinebuf(logfile);
	return logfile;
}

/* logfile == NULL is ok */
void close_log(FILE* logfile)
{
	if (!logfile)
		return;
	PARA_INFO_LOG("%s", "closing logfile\n");
	fclose(logfile);
}

void log_welcome(const char *whoami, int loglevel)
{
	PARA_INFO_LOG("welcome to %s " VERSION " ("BUILD_DATE")\n", whoami);
	PARA_DEBUG_LOG("using loglevel %d\n", loglevel);
}
void para_drop_privileges(const char *username)
{
	struct passwd *p;
	char *tmp;

	if (geteuid())
		return;
	if (!username) {
		PARA_EMERG_LOG("%s", "root privileges, but no user option given\n");
		exit(EXIT_FAILURE);
	}
	tmp = para_strdup(username);
	p = getpwnam(tmp);
	free(tmp);
	if (!p) {
		PARA_EMERG_LOG("%s", "no such user\n");
		exit(EXIT_FAILURE);
	}
	PARA_NOTICE_LOG("%s", "dropping root privileges\n");
	setuid(p->pw_uid);
	PARA_DEBUG_LOG("uid: %d, euid: %d\n", getuid(), geteuid());
	setuid(p->pw_uid);
}

/*
 * set or get server's uptime
 */
int server_uptime(int set_or_get)
{
	static time_t startuptime;
	time_t now;

	if (set_or_get == SET) {
		time(&startuptime);
		return 0;
	}
	time(&now);
	return((int) difftime(now, startuptime));
}

/*
 * construct string containing uptime
 */
char *uptime_str(int i)
{
	return make_message("%i:%02i:%02i", i / 86400,
		(i / 3600) % 24, (i / 60) % 60);
}

